#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_vnic_template
short_description: Manage vNIC Templates for Cisco Intersight
description:
  - Create, update, and delete vNIC Templates on Cisco Intersight.
  - vNIC Templates define network interface configurations that can be used by LAN Connectivity policies.
  - Templates provide a standardized way to configure vNICs with consistent network and adapter policies.
  - vNIC Templates are only applicable for FI-Attached (Fabric Interconnect) deployments.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/VnicTemplate/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the vNIC Template.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the vNIC Template.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  enable_override:
    description:
      - When enabled, the configuration of the derived instances may override the template configuration.
    type: bool
    default: false
  switch_id:
    description:
      - The fabric port to which the vNIC will be associated.
    type: str
    choices: ['A', 'B']
    default: 'A'
  failover_enabled:
    description:
      - Enables automatic vNIC failover to the secondary Fabric Interconnect
        if the primary path fails.
      - Failover applies only to Cisco VICs that are connected to a Fabric Interconnect cluster.
    type: bool
    default: false
  cdn_source:
    description:
      - Source of the CDN. It can either be user specified or be the same as the vNIC name.
    type: str
    choices: ['vnic', 'user']
    default: 'vnic'
  cdn_value:
    description:
      - CDN value when cdn_source is 'user'.
    type: str
  mac_pool_name:
    description:
      - The MAC pool that is assigned to the vNIC Template.
      - Required when state is 'present'.
    type: str
  fabric_eth_network_group_policy_name:
    description:
      - Relationship to the Fabric Ethernet Group Policy.
      - Required when state is 'present'.
    type: str
  fabric_eth_network_control_policy_name:
    description:
      - Relationship to the Fabric Ethernet Network Control Policy.
      - Required when state is 'present'.
    type: str
  eth_qos_policy_name:
    description:
      - Relationship to the Ethernet QoS Policy.
      - Required when state is 'present'.
    type: str
  eth_adapter_policy_name:
    description:
      - Relationship to the Ethernet Adapter Policy.
      - Required when state is 'present'.
    type: str
  iscsi_boot_policy_name:
    description:
      - Relationship to the boot iSCSI Policy.
    type: str
  pin_group_name:
    description:
      - Pingroup name associated to vNIC for static pinning.
      - LCP deploy will resolve pingroup name and fetches the corresponding uplink port/port channel to pin the vNIC traffic.
    type: str
  connection_type:
    description:
      - Type of connection for the vNIC.
    type: str
    choices: ['none', 'usnic', 'vmq', 'sriov']
    default: 'none'
  usnic_settings:
    description:
      - USNIC settings when connection_type is 'usnic'.
      - Required when connection_type is 'usnic'.
    type: dict
    suboptions:
      count:
        description:
          - Number of usNIC interfaces to be created.
          - When usNIC is enabled, the valid values are from 1 to 225.
          - When usNIC is disabled, the default value is 0.
        type: int
        default: 0
      cos:
        description:
          - Class of Service to be used for traffic on the usNIC.
        type: int
        choices: [0, 1, 2, 3, 4, 5, 6]
        default: 5
      usnic_adapter_policy_name:
        description:
          - Ethernet Adapter policy to be associated with the usNICs.
          - Required when connection_type is 'usnic'.
        type: str
  vmq_settings:
    description:
      - VMQ settings when connection_type is 'vmq'.
    type: dict
    suboptions:
      enabled:
        description:
          - Enable VMQ.
        type: bool
        default: true
      multi_queue_support:
        description:
          - Enables Virtual Machine Multi-Queue feature on the virtual interface.
          - VMMQ allows configuration of multiple I/O queues for a single VM and thus distributes traffic across multiple CPU cores in a VM.
        type: bool
        default: false
      num_interrupts:
        description:
          - The number of interrupt resources to be allocated.
          - Recommended value is the number of CPU threads or logical processors available in the server.
        type: int
        default: 16
      num_vmqs:
        description:
          - The number of hardware Virtual Machine Queues to be allocated.
          - The number of VMQs per adapter must be one more than the maximum number of VM NICs.
        type: int
        default: 4
      num_sub_vnics:
        description:
          - Number of sub vNICs (0-64).
          - Only applicable when multi_queue_support is true.
        type: int
        default: 64
      vmmq_adapter_policy_name:
        description:
          - Name of the VMMQ Adapter Policy.
          - Only applicable when multi_queue_support is true.
        type: str
  sriov_settings:
    description:
      - SR-IOV settings when connection_type is 'sriov'.
    type: dict
    suboptions:
      enabled:
        description:
          - Enable SR-IOV.
        type: bool
        default: true
      vf_count:
        description:
          - Number of Virtual Functions (1-64).
        type: int
        default: 64
      rx_count_per_vf:
        description:
          - Receive Queue Count per VF (1-8).
        type: int
        default: 4
      tx_count_per_vf:
        description:
          - Transmit Queue Count per VF (1-8).
        type: int
        default: 1
      comp_count_per_vf:
        description:
          - Completion Queue Count per VF (1-16).
        type: int
        default: 5
      int_count_per_vf:
        description:
          - Interrupt Count per VF (1-16).
        type: int
        default: 8
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a basic vNIC Template
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "basic-vnic-template"
    description: "Basic vNIC template for production servers"
    enable_override: false
    switch_id: "A"
    failover_enabled: false
    cdn_source: "vnic"
    mac_pool_name: "default-mac-pool"
    fabric_eth_network_group_policy_name: "default-network-group"
    fabric_eth_network_control_policy_name: "default-network-control"
    eth_qos_policy_name: "default-qos-policy"
    eth_adapter_policy_name: "default-adapter-policy"
    connection_type: "none"
    tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Site"
        Value: "DataCenter-A"
    state: present

- name: Create a vNIC Template with custom CDN
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "custom-cdn-template"
    description: "vNIC template with custom CDN value"
    enable_override: true
    switch_id: "B"
    failover_enabled: true
    cdn_source: "user"
    cdn_value: "Management-NIC"
    mac_pool_name: "mgmt-mac-pool"
    fabric_eth_network_group_policy_name: "mgmt-network-group"
    fabric_eth_network_control_policy_name: "mgmt-network-control"
    eth_qos_policy_name: "mgmt-qos-policy"
    eth_adapter_policy_name: "mgmt-adapter-policy"
    iscsi_boot_policy_name: "mgmt-iscsi-boot-policy"
    connection_type: "none"
    state: present

- name: Create a vNIC Template with USNIC
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "usnic-template"
    description: "vNIC template with USNIC configuration"
    enable_override: false
    switch_id: "A"
    mac_pool_name: "hpc-mac-pool"
    fabric_eth_network_group_policy_name: "hpc-network-group"
    fabric_eth_network_control_policy_name: "hpc-network-control"
    eth_qos_policy_name: "hpc-qos-policy"
    eth_adapter_policy_name: "hpc-adapter-policy"
    connection_type: "usnic"
    usnic_settings:
      count: 0
      cos: 5
      usnic_adapter_policy_name: "hpc-adapter-policy"
    state: present

- name: Create a vNIC Template with VMQ
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "vmq-template"
    description: "vNIC template with VMQ configuration"
    enable_override: false
    switch_id: "A"
    mac_pool_name: "vm-mac-pool"
    fabric_eth_network_group_policy_name: "vm-network-group"
    fabric_eth_network_control_policy_name: "vm-network-control"
    eth_qos_policy_name: "vm-qos-policy"
    eth_adapter_policy_name: "vm-adapter-policy"
    connection_type: "vmq"
    vmq_settings:
      enabled: true
      multi_queue_support: false
      num_interrupts: 16
      num_vmqs: 4
    state: present

- name: Create a vNIC Template with VMQ multi-queue support
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "vmq-multiqueue-template"
    description: "vNIC template with VMQ multi-queue configuration"
    enable_override: true
    switch_id: "B"
    mac_pool_name: "vm-mac-pool"
    fabric_eth_network_group_policy_name: "vm-network-group"
    fabric_eth_network_control_policy_name: "vm-network-control"
    eth_qos_policy_name: "vm-qos-policy"
    eth_adapter_policy_name: "vm-adapter-policy"
    connection_type: "vmq"
    vmq_settings:
      enabled: true
      multi_queue_support: true
      num_sub_vnics: 64
      vmmq_adapter_policy_name: "vmmq-adapter-policy"
    state: present

- name: Create a vNIC Template with SR-IOV
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "sriov-template"
    description: "vNIC template with SR-IOV configuration"
    enable_override: false
    switch_id: "A"
    mac_pool_name: "sriov-mac-pool"
    fabric_eth_network_group_policy_name: "sriov-network-group"
    fabric_eth_network_control_policy_name: "sriov-network-control"
    eth_qos_policy_name: "sriov-qos-policy"
    eth_adapter_policy_name: "sriov-adapter-policy"
    connection_type: "sriov"
    sriov_settings:
      enabled: true
      vf_count: 64
      rx_count_per_vf: 4
      tx_count_per_vf: 1
      comp_count_per_vf: 5
      int_count_per_vf: 8
    state: present

- name: Create a vNIC Template with pin group
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "vnic-pinned-template"
    description: "vNIC template with static pinning"
    enable_override: false
    switch_id: "A"
    failover_enabled: false
    mac_pool_name: "default-mac-pool"
    fabric_eth_network_group_policy_name: "default-network-group"
    fabric_eth_network_control_policy_name: "default-network-control"
    eth_qos_policy_name: "default-qos-policy"
    eth_adapter_policy_name: "default-adapter-policy"
    pin_group_name: "pingroup-a"
    connection_type: "none"
    state: present

- name: Delete a vNIC Template
  cisco.intersight.intersight_vnic_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "old-vnic-template"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "test-vnic-template",
        "ObjectType": "vnic.VnicTemplate",
        "EnableOverride": false,
        "SwitchId": "A",
        "FailoverEnabled": false,
        "Cdn": {
            "Source": "vnic"
        },
        "MacPool": {
            "Name": "default-mac-pool",
            "ObjectType": "macpool.Pool"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec
from ansible_collections.cisco.intersight.plugins.module_utils.intersight_vnic_utils import (
    validate_cdn_config, validate_usnic_settings, validate_vmq_settings, validate_sriov_settings,
    build_cdn_config, build_connection_settings, resolve_policy_moids_from_mappings,
    get_common_policy_mappings, get_iscsi_boot_policy_mapping, get_fabric_network_policy_mappings,
    get_mac_pool_policy_mapping, get_common_settings_argument_spec
)


def get_template_policy_mappings(module):
    """
    Get policy mappings for vNIC Template configuration.
    """
    # Start with common policies
    policy_mappings = get_common_policy_mappings()

    # Add fabric network policies (vNIC templates are FI-attached only)
    policy_mappings.update(get_fabric_network_policy_mappings())

    # Add MAC pool (always required for templates)
    policy_mappings.update(get_mac_pool_policy_mapping())

    # iSCSI boot policy is optional - only add if specified
    if module.params.get('iscsi_boot_policy_name'):
        policy_mappings.update(get_iscsi_boot_policy_mapping())

    return policy_mappings


def resolve_policy_moids(intersight, policy_cache, module):
    """
    Resolve all policy MOIDs for vNIC Template configuration.
    """
    policy_mappings = get_template_policy_mappings(module)
    organization_name = module.params['organization']
    return resolve_policy_moids_from_mappings(intersight, policy_cache, module, module.params, policy_mappings, organization_name)


def validate_input(module):
    """
    Validate module input parameters
    """
    if module.params['state'] == 'present':
        # Validate required fields for vNIC Template creation
        required_fields = [
            'mac_pool_name',
            'fabric_eth_network_group_policy_name',
            'fabric_eth_network_control_policy_name',
            'eth_qos_policy_name',
            'eth_adapter_policy_name'
        ]
        for field in required_fields:
            if not module.params.get(field):
                module.fail_json(msg=f"{field} is required when state is 'present'")

        # Validate CDN configuration
        validate_cdn_config(module, module.params)

        connection_type = module.params.get('connection_type', 'none')

        # Validate connection type specific settings
        template_name = module.params.get('name', 'unknown')
        if connection_type == 'usnic':
            usnic_settings = module.params.get('usnic_settings')
            validate_usnic_settings(module, usnic_settings, template_name)
        elif connection_type == 'vmq':
            vmq_settings = module.params.get('vmq_settings', {})
            validate_vmq_settings(module, vmq_settings, template_name)
        elif connection_type == 'sriov':
            sriov_settings = module.params.get('sriov_settings', {})
            validate_sriov_settings(module, sriov_settings, template_name)


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enable_override=dict(type='bool', default=False),
        eth_qos_policy_name=dict(type='str'),
        eth_adapter_policy_name=dict(type='str'),
        pin_group_name=dict(type='str'),
    )
    # Add connection settings argument specs
    argument_spec.update(get_common_settings_argument_spec())

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    if module.params['state'] == 'present':
        validate_input(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure vNIC Template
    resource_path = '/vnic/VnicTemplates'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Add vNIC Template specific parameters
        intersight.api_body['EnableOverride'] = intersight.module.params['enable_override']
        intersight.api_body['SwitchId'] = intersight.module.params['switch_id']
        intersight.api_body['FailoverEnabled'] = intersight.module.params['failover_enabled']

        # Add CDN configuration
        intersight.api_body['Cdn'] = build_cdn_config(intersight.module.params)

        # Cache for policy MOIDs to avoid redundant API calls
        policy_cache = {}

        # Resolve and add policy MOIDs
        policy_moids = resolve_policy_moids(intersight, policy_cache, intersight.module)
        intersight.api_body.update(policy_moids)

        # Add connection type specific settings
        organization_name = intersight.module.params['organization']
        connection_settings = build_connection_settings(intersight, policy_cache, intersight.module, intersight.module.params, organization_name)
        intersight.api_body.update(connection_settings)

        # Add pin group name if specified
        if intersight.module.params.get('pin_group_name'):
            intersight.api_body['PinGroupName'] = intersight.module.params['pin_group_name']

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_network_feed
short_description: Manages network-feed objects on Checkpoint over Web Services API
description:
  - Manages network-feed objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R81.20 management version.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  feed_url:
    description:
      - URL of the feed. URL should be written as http or https.
    type: str
  certificate_id:
    description:
      - Certificate SHA-1 fingerprint to access the feed.
    type: str
  feed_format:
    description:
      - Feed file format.
    type: str
    choices: ['Flat List', 'JSON']
  feed_type:
    description:
      - Feed type to be enforced.
    type: str
    choices: ['Domain', 'IP Address', 'IP Address/Domain']
  password:
    description:
      - password for authenticating with the URL.
    type: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  username:
    description:
      - username for authenticating with the URL.
    type: str
  custom_header:
    description:
      - Headers to allow different authentication methods with the URL.
    type: list
    elements: dict
    suboptions:
      header_name:
        description:
          - The name of the HTTP header we wish to add.
        type: str
      header_value:
        description:
          - The name of the HTTP value we wish to add.
        type: str
  update_interval:
    description:
      - Interval in minutes for updating the feed on the Security Gateway.
    type: int
  data_column:
    description:
      - Number of the column that contains the feed's data.
    type: int
  fields_delimiter:
    description:
      - The delimiter that separates between the columns in the feed.
    type: str
  ignore_lines_that_start_with:
    description:
      - A prefix that will determine which lines to ignore.
    type: str
  json_query:
    description:
      - JQ query to be parsed.
    type: str
  use_gateway_proxy:
    description:
      - Use the gateway's proxy for retrieving the feed.
    type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-network-feed
  cp_mgmt_network_feed:
    custom_header:
      - header_name: header1
        header_value: value1
      - header_name: header2
        header_value: value2
    data_column: 1
    feed_format: Flat List
    feed_type: IP Address
    feed_url: https://www.feedsresource.com/resource
    fields_delimiter: "\t"
    ignore_lines_that_start_with: '!'
    name: network_feed
    password: feed_password
    state: present
    update_interval: 60
    use_gateway_proxy: false
    username: feed_username

- name: set-network-feed
  cp_mgmt_network_feed:
    custom_header:
      - header_name: new_header
        header_value: new_value
    data_column: 1
    feed_format: Flat List
    feed_type: IP Address
    feed_url: https://www.feedsresource.com/new_resource
    fields_delimiter: ','
    ignore_lines_that_start_with: '!'
    name: network_feed
    password: new_password
    state: present
    update_interval: 60
    use_gateway_proxy: false
    username: new_username

- name: delete-network-feed
  cp_mgmt_network_feed:
    name: network_feed
    state: absent
"""

RETURN = """
cp_mgmt_network_feed:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        feed_url=dict(type="str"),
        certificate_id=dict(type="str"),
        feed_format=dict(type="str", choices=["Flat List", "JSON"]),
        feed_type=dict(
            type="str", choices=["Domain", "IP Address", "IP Address/Domain"]
        ),
        password=dict(type="str", no_log=True),
        tags=dict(type="list", elements="str"),
        username=dict(type="str"),
        custom_header=dict(
            type="list",
            elements="dict",
            options=dict(
                header_name=dict(type="str"), header_value=dict(type="str")
            ),
        ),
        update_interval=dict(type="int"),
        data_column=dict(type="int"),
        fields_delimiter=dict(type="str"),
        ignore_lines_that_start_with=dict(type="str"),
        json_query=dict(type="str"),
        use_gateway_proxy=dict(type="bool"),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        domains_to_process=dict(type="list", elements="str"),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "network-feed"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
"""
The module file for ios_vlans
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_vlans
short_description: Resource module to configure VLANs.
description:
  This module provides declarative management of VLANs on Cisco IOS network
  devices.
version_added: 1.0.0
author:
  - Sumit Jaiswal (@justjais)
  - Sagar Paul (@KB-perByte)
  - Padmini Priyadarshini Sivaraj (@PadminiSivaraj)
notes:
  - Tested against Cisco IOS-XE device with Version 17.13.01 on Cat9k on CML.
  - Starting from v2.5.0, this module will fail when run against Cisco IOS devices that do
    not support VLANs. The offline states (C(rendered) and C(parsed)) will work as expected.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  config:
    description: A dictionary of VLANs options
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Ascii name of the VLAN.
          - NOTE, I(name) should not be named/appended with I(default) as it is reserved
            for device default vlans.
        type: str
      vlan_id:
        description:
          - ID of the VLAN. Range 1-4094
        type: int
        required: true
      mtu:
        description:
          - VLAN Maximum Transmission Unit.
          - Refer to vendor documentation for valid values.
          - This option is DEPRECATED use ios_interfaces to configure mtu,
            this attribute will be removed after 2027-01-01.
          - mtu is collected as part of facts,
            but a mtu command wont be fired if the configuration
            is present in the playbook.
        type: int
      state:
        description:
          - Operational state of the VLAN
        type: str
        choices:
          - active
          - suspend
      remote_span:
        description:
          - Configure as Remote SPAN VLAN
        type: bool
      shutdown:
        description:
          - Specifies whether VLAN switching should be administratively enabled or disabled.
          - When set to C(enabled), the VLAN interface is administratively shut down, which prevents it from forwarding traffic.
          - When set to C(disabled), the VLAN interface is administratively enabled by issuing the internal C(no shutdown) command,
            allowing it to forward traffic.
          - The operational state of the VLAN depends on both the administrative state (C(shutdown) or C(no shutdown)) and the physical link status.
        type: str
        choices:
          - enabled
          - disabled
      private_vlan:
        description:
          - Options for private vlan configuration.
        type: dict
        suboptions:
          type:
            description:
              - Private VLAN type
            type: str
            choices:
              - primary
              - isolated
              - community
          associated:
            description:
              - "List of private VLANs associated with the primary . Only works with `type: primary`."
            type: list
            elements: int
      member:
        description:
          - Members of VLAN
        type: dict
        suboptions:
          vni:
            description:
              - VXLAN vni
            type: int
            required: true
          evi:
            description:
              - Ethernet Virtual Private Network (EVPN)
            type: int
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device
        by executing the command B(show vlan).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of commands I(show vlan) and I(show
        running-config | sec ^vlan configuration .+) executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - purged
      - parsed
    default: merged
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

- name: Merge provided configuration with device configuration
  cisco.ios.ios_vlans:
    config:
      - name: Vlan_10
        vlan_id: 10
        state: active
        shutdown: disabled
        remote_span: true
      - name: Vlan_20
        vlan_id: 20
        mtu: 610
        state: active
        shutdown: enabled
      - name: Vlan_30
        vlan_id: 30
        state: suspend
        shutdown: enabled
    state: merged

# After state:
# ------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 10   vlan_10                          active
# 20   vlan_20                          act/lshut
# 30   vlan_30                          sus/lshut
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 10   enet  100010     1500  -      -      -        -    -        0      0
# 20   enet  100020     610   -      -      -        -    -        0      0
# 30   enet  100030     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0
#
# Remote SPAN VLANs
# ------------------------------------------------------------------------------
# 10

# Using merged

# Before state:
# -------------
#
# Leaf-01#show run nve | sec ^vlan configuration
# vlan configuration 101
#  member evpn-instance 101 vni 10101
# vlan configuration 201
#  member evpn-instance 201 vni 10201


- name: Merge provided configuration with device configuration
  cisco.ios.ios_vlans:
    config:
      - vlan_id: 102
        member:
          vni: 10102
          evi: 102
      - vlan_id: 901
        member:
          vni: 50901
    state: merged

# After state:
# ------------
#
# Leaf-01#show run nve | sec ^vlan configuration
# vlan configuration 101
#  member evpn-instance 101 vni 10101
# vlan configuration 102
#  member evpn-instance 102 vni 10102
# vlan configuration 201
#  member evpn-instance 201 vni 10201
# vlan configuration 901
#  member vni 50901

# Using overridden

# Before state:
# -------------
#
# S1#show vlan

# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/0, Gi0/1, Gi0/2, Gi0/3
# 10   Vlan_10                          active
# 20   Vlan_20                          active
# 30   Vlan_30                          suspended
# 44   Vlan_44                          suspended
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup

# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 10   enet  100010     1500  -      -      -        -    -        0      0
# 20   enet  100020     610   -      -      -        -    -        0      0
# 30   enet  100030     1500  -      -      -        -    -        0      0
# 44   enet  100044     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

# Remote SPAN VLANs
# ------------------------------------------------------------------------------


# Primary Secondary Type              Ports
# ------- --------- ----------------- ------------------------------------------
# S1#


- name: Override device configuration of all VLANs with provided configuration
  cisco.ios.ios_vlans:
    config:
      - name: Vlan_2020
        state: active
        vlan_id: 20
        shutdown: disabled
    state: overridden

# Task output:
# ------------

# after:
# - mtu: 1500
#   name: default
#   shutdown: disabled
#   state: active
#   vlan_id: 1
# - mtu: 1500
#   name: Vlan_2020
#   shutdown: disabled
#   state: active
#   vlan_id: 20
# - mtu: 1500
#   name: fddi-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1002
# - mtu: 1500
#   name: token-ring-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1003
# - mtu: 1500
#   name: fddinet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1004
# - mtu: 1500
#   name: trnet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1005

# before:
# - mtu: 1500
#   name: default
#   shutdown: disabled
#   state: active
#   vlan_id: 1
# - mtu: 1500
#   name: Vlan_10
#   shutdown: disabled
#   state: active
#   vlan_id: 10
# - mtu: 610
#   name: Vlan_20
#   shutdown: disabled
#   state: active
#   vlan_id: 20
# - mtu: 1500
#   name: Vlan_30
#   shutdown: disabled
#   state: suspend
#   vlan_id: 30
# - mtu: 1500
#   name: Vlan_44
#   shutdown: disabled
#   state: suspend
#   vlan_id: 44
# - mtu: 1500
#   name: fddi-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1002
# - mtu: 1500
#   name: token-ring-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1003
# - mtu: 1500
#   name: fddinet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1004
# - mtu: 1500
#   name: trnet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1005

# commands:
# - no vlan 1
# - no vlan 10
# - no vlan 30
# - no vlan 44
# - no vlan 1002
# - no vlan 1003
# - no vlan 1004
# - no vlan 1005
# - vlan 20
# - name Vlan_2020
# - no mtu 610
# - no vlan configuration 1
# - no vlan configuration 10
# - no vlan configuration 30
# - no vlan configuration 44
# - no vlan configuration 1002
# - no vlan configuration 1003
# - no vlan configuration 1004
# - no vlan configuration 1005

# After state:
# ------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/0, Gi0/1, Gi0/2, Gi0/3
# 20   Vlan_2020                        active
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup

# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 20   enet  100020     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

# Remote SPAN VLANs
# ------------------------------------------------------------------------------


# Primary Secondary Type              Ports
# ------- --------- ----------------- ------------------------------------------

# Using purged

# Before state:
# -------------
#
# S1#show vlan

# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/0, Gi0/1, Gi0/2, Gi0/3
# 20   Vlan_2020                        active
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup

# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 20   enet  100020     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

# Remote SPAN VLANs
# ------------------------------------------------------------------------------


# Primary Secondary Type              Ports
# ------- --------- ----------------- ------------------------------------------

# S1#show running-config | section ^vlan configuration .+
# vlan configuration 20


- name: Purge all vlans configuration
  cisco.ios.ios_vlans:
    config:
    state: purged

# Task output:
# ------------

# after:
# - mtu: 1500
#   name: default
#   shutdown: disabled
#   state: active
#   vlan_id: 1
# - mtu: 1500
#   name: fddi-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1002
# - mtu: 1500
#   name: token-ring-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1003
# - mtu: 1500
#   name: fddinet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1004
# - mtu: 1500
#   name: trnet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1005

# before:
# - mtu: 1500
#   name: default
#   shutdown: disabled
#   state: active
#   vlan_id: 1
# - mtu: 1500
#   name: Vlan_2020
#   shutdown: disabled
#   state: active
#   vlan_id: 20
# - mtu: 1500
#   name: fddi-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1002
# - mtu: 1500
#   name: token-ring-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1003
# - mtu: 1500
#   name: fddinet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1004
# - mtu: 1500
#   name: trnet-default
#   shutdown: enabled
#   state: active
#   vlan_id: 1005

# commands:
# - no vlan 1
# - no vlan 20
# - no vlan 1002
# - no vlan 1003
# - no vlan 1004
# - no vlan 1005
# - no vlan configuration 1
# - no vlan configuration 20
# - no vlan configuration 1002
# - no vlan configuration 1003
# - no vlan configuration 1004
# - no vlan configuration 1005

# After state:
# ------------
#
# S1#show vlan

# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/0, Gi0/1, Gi0/2, Gi0/3
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup

# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

# Remote SPAN VLANs
# ------------------------------------------------------------------------------


# Primary Secondary Type              Ports
# ------- --------- ----------------- ------------------------------------------

# S1#show running-config | section ^vlan configuration .+
# S1#


# Using replaced

# Before state:
# -------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 10   vlan_10                          active
# 20   vlan_20                          act/lshut
# 30   vlan_30                          sus/lshut
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 10   enet  100010     1500  -      -      -        -    -        0      0
# 20   enet  100020     610   -      -      -        -    -        0      0
# 30   enet  100030     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0
#
# Remote SPAN VLANs
# ------------------------------------------------------------------------------
# 10

- name: Replaces device configuration of listed VLANs with provided configuration
  cisco.ios.ios_vlans:
    config:
      - vlan_id: 20
        name: Test_VLAN20
        mtu: 700
        shutdown: disabled
      - vlan_id: 50
        name: pvlan-isolated
        private_vlan:
          type: isolated
      - vlan_id: 60
        name: pvlan-community
        private_vlan:
          type: community
      - vlan_id: 70
        name: pvlan-primary
        private_vlan:
          type: primary
          associated:
            - 50
            - 60

    state: replaced

# After state:
# ------------
#
# vios_l2#sh vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/0, Gi0/1, Gi0/2, Gi0/3
# 10   Vlan_10                          active
# 20   Test_VLAN20                      active
# 50   pvlan-isolated                   active
# 60   pvlan-community                  active
# 70   pvlan-primary                    active
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 10   enet  100010     1000  -      -      -        -    -        0      0
# 20   enet  100020     700   -      -      -        -    -        0      0
# 50   enet  100050     1500  -      -      -        -    -        0      0
# 60   enet  100051     1500  -      -      -        -    -        0      0
# 70   enet  100059     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0
#
# Remote SPAN VLANs
# ------------------------------------------------------------------------------
#
#
# Primary Secondary Type              Ports
# ------- --------- ----------------- ------------------------------------------
# 70      50        isolated
# 70      60        community

# Using deleted

# Before state:
# -------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 10   vlan_10                          active
# 20   vlan_20                          act/lshut
# 30   vlan_30                          sus/lshut
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 10   enet  100010     1500  -      -      -        -    -        0      0
# 20   enet  100020     610   -      -      -        -    -        0      0
# 30   enet  100030     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0
#
# Remote SPAN VLANs
# ------------------------------------------------------------------------------
# 10

- name: Delete attributes of given VLANs
  cisco.ios.ios_vlans:
    config:
      - vlan_id: 10
      - vlan_id: 20
    state: deleted

# After state:
# -------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 30   vlan_30                          sus/lshut
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 30   enet  100030     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

# Using deleted

# Before state:
# -------------
#
# Leaf-01#show run nve | sec ^vlan configuration
# vlan configuration 101
#  member evpn-instance 101 vni 10101
# vlan configuration 102
#  member evpn-instance 102 vni 10102
# vlan configuration 201
#  member evpn-instance 201 vni 10201
# vlan configuration 901
#  member vni 50901

- name: Delete attributes of given VLANs
  cisco.ios.ios_vlans:
    config:
      - vlan_id: 101
    state: deleted

# After state:
# -------------
#
# Leaf-01#show run nve | sec ^vlan configuration
# vlan configuration 101
# vlan configuration 102
#  member evpn-instance 102 vni 10102
# vlan configuration 201
#  member evpn-instance 201 vni 10201
# vlan configuration 901
#  member vni 50901

# Using Deleted without any config passed
# "(NOTE: This will delete all of configured vlans attributes)"

# Before state:
# -------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 10   vlan_10                          active
# 20   vlan_20                          act/lshut
# 30   vlan_30                          sus/lshut
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 10   enet  100010     1500  -      -      -        -    -        0      0
# 20   enet  100020     610   -      -      -        -    -        0      0
# 30   enet  100030     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0
#
# Remote SPAN VLANs
# ------------------------------------------------------------------------------
# 10

- name: Delete attributes of ALL VLANs
  cisco.ios.ios_vlans:
    state: deleted

# After state:
# -------------
#
# vios_l2#show vlan
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

# Using Deleted without any config passed
# "(NOTE: This will delete all of configured vlans attributes)"

# Before state:
# -------------
#
# Leaf-01#show run nve | sec ^vlan configuration
# vlan configuration 101
#  member evpn-instance 101 vni 10101
# vlan configuration 102
#  member evpn-instance 102 vni 10102
# vlan configuration 201
#  member evpn-instance 201 vni 10201
# vlan configuration 202
#  member evpn-instance 202 vni 10202
# vlan configuration 901
#  member vni 50901

- name: Delete attributes of ALL VLANs
  cisco.ios.ios_vlans:
    state: deleted

# After state:
# -------------
#
# Leaf-01#show run nve | sec ^vlan configuration
# no vlan configuration 101
# no vlan configuration 102
# no vlan configuration 201
# no vlan configuration 202
# no vlan configuration 901
# no vlan configuration 902

# Using gathered, vlan configuration only

# Before state:
# -------------
#
# Leaf-01#show run nve | sec ^vlan configuration
# vlan configuration 101
#  member evpn-instance 101 vni 10101
# vlan configuration 102
#  member evpn-instance 102 vni 10102
# vlan configuration 201
#  member evpn-instance 201 vni 10201
# vlan configuration 202
#  member evpn-instance 202 vni 10202
# vlan configuration 901
#  member vni 50901

- name: Gather listed vlans with provided configurations
  cisco.ios.ios_vlans:
    state: gathered

# Module Execution Result:
# ------------------------
#
# gathered = [
#     {
#         "member": {
#             "evi": 101,
#             "vni": 10101
#         },
#         "vlan_id": 101
#     },
#     {
#         "member": {
#             "evi": 102,
#             "vni": 10102
#         },
#         "vlan_id": 102
#     },
#     {
#         "member": {
#             "evi": 201,
#             "vni": 10201
#         },
#         "vlan_id": 201
#     },
#     {
#         "member": {
#             "evi": 202,
#             "vni": 10202
#         },
#         "vlan_id": 202
#     },
#     {
#         "member": {
#             "vni": 50901
#         },
#         "vlan_id": 901
#     },
#     {
#         "member": {
#             "vni": 50902
#         },
#         "vlan_id": 902
#     }
# ]

# Using Rendered

- name: Render the commands for provided  configuration
  cisco.ios.ios_vlans:
    config:
      - name: Vlan_10
        vlan_id: 10
        state: active
        shutdown: disabled
        remote_span: true
      - name: Vlan_20
        vlan_id: 20
        mtu: 610
        state: active
        shutdown: enabled
      - name: Vlan_30
        vlan_id: 30
        state: suspend
        shutdown: enabled
    state: rendered

# Module Execution Result:
# ------------------------
#
# "rendered": [
#         "vlan 10",
#         "name Vlan_10",
#         "state active",
#         "remote-span",
#         "no shutdown",
#         "vlan 20",
#         "name Vlan_20",
#         "state active",
#         "mtu 610",
#         "shutdown",
#         "vlan 30",
#         "name Vlan_30",
#         "state suspend",
#         "shutdown"
#     ]

# Using Rendered

- name: Render the commands for provided configuration
  cisco.ios.ios_vlans:
    config:
      - vlan_id: 101
        member:
          vni: 10101
          evi: 101
      - vlan_id: 102
        member:
          vni: 10102
          evi: 102
    state: rendered

# Module Execution Result:
# ------------------------
#
# "rendered": [
#     "vlan configuration 101",
#     "member evpn-instance 101 vni 10101",
#     "vlan configuration 102",
#     "member evpn-instance 102 vni 10102"
# ]

# Using Parsed

# File: parsed.cfg
# ----------------
#
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 10   vlan_10                          active
# 20   vlan_20                          act/lshut
# 30   vlan_30                          sus/lshut
# 1002 fddi-default                     act/unsup
# 1003 token-ring-default               act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trnet-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 10   enet  100010     1500  -      -      -        -    -        0      0
# 20   enet  100020     1500  -      -      -        -    -        0      0
# 30   enet  100030     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 tr    101003     1500  -      -      -        -    -        0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trnet 101005     1500  -      -      -        ibm  -        0      0

- name: Parse the commands for provided configuration
  cisco.ios.ios_vlans:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------
#
# "parsed": [
#         {
#             "mtu": 1500,
#             "name": "default",
#             "shutdown": "disabled",
#             "state": "active",
#             "vlan_id": 1
#         },
#         {
#             "mtu": 1500,
#             "name": "vlan_10",
#             "shutdown": "disabled",
#             "state": "active",
#             "vlan_id": 10
#         },
#         {
#             "mtu": 1500,
#             "name": "vlan_20",
#             "shutdown": "enabled",
#             "state": "active",
#             "vlan_id": 20
#         },
#         {
#             "mtu": 1500,
#             "name": "vlan_30",
#             "shutdown": "enabled",
#             "state": "suspend",
#             "vlan_id": 30
#         },
#         {
#             "mtu": 1500,
#             "name": "fddi-default",
#             "shutdown": "enabled",
#             "state": "active",
#             "vlan_id": 1002
#         },
#         {
#             "mtu": 1500,
#             "name": "token-ring-default",
#             "shutdown": "enabled",
#             "state": "active",
#             "vlan_id": 1003
#         },
#         {
#             "mtu": 1500,
#             "name": "fddinet-default",
#             "shutdown": "enabled",
#             "state": "active",
#             "vlan_id": 1004
#         },
#         {
#             "mtu": 1500,
#             "name": "trnet-default",
#             "shutdown": "enabled",
#             "state": "active",
#             "vlan_id": 1005
#         }
#     ]

# Using Parsed Vlan configuration only

# File: parsed.cfg
# ----------------
#
# vlan configuration 101
#  member evpn-instance 101 vni 10101
# vlan configuration 102
#  member evpn-instance 102 vni 10102
# vlan configuration 901
#  member vni 50901

- name: Parse the commands for provided configuration
  cisco.ios.ios_vlans:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------
#
# "parsed": [
#     {
#         "member": {
#             "evi": 101,
#             "vni": 10101
#         },
#         "vlan_id": 101
#     },
#     {
#         "member": {
#             "evi": 102,
#             "vni": 10102
#         },
#         "vlan_id": 102
#     },
#     {
#         "member": {
#             "vni": 50901
#         },
#         "vlan_id": 901
#     }
# ]

# Using Parsed, Vlan and vlan configuration

# File: parsed.cfg
# ----------------
#
# VLAN Name                             Status    Ports
# ---- -------------------------------- --------- -------------------------------
# 1    default                          active    Gi0/1, Gi0/2
# 101  RemoteIsInMyName                 act/unsup Fa0/1, Fa0/4, Fa0/5, Fa0/6, Fa0/7, Fa0/8, Fa0/9, Fa0/10, Fa0/11, Fa0/12
#                                                 Fa0/13, Fa0/14, Fa0/15, Fa0/16, Fa0/17, Fa0/18, Fa0/19, Fa0/20, Fa0/21
#                                                 Fa0/22, Fa0/23, Fa0/24, Fa0/25, Fa0/26, Fa0/27, Fa0/28, Fa0/29, Fa0/30
#                                                 Fa0/31, Fa0/32, Fa0/33, Fa0/34, Fa0/35, Fa0/36, Fa0/37, Fa0/38, Fa0/39
#                                                 Fa0/40, Fa0/41, Fa0/42, Fa0/43, Fa0/44, Fa0/45, Fa0/46, Fa0/47, Fa0/48
# 150  VLAN0150                         active
# 888  a_very_long_vlan_name_a_very_long_vlan_name
#                                     active
# 1002 fddi-default                     act/unsup
# 1003 trcrf-default                    act/unsup
# 1004 fddinet-default                  act/unsup
# 1005 trbrf-default                    act/unsup
#
# VLAN Type  SAID       MTU   Parent RingNo BridgeNo Stp  BrdgMode Trans1 Trans2
# ---- ----- ---------- ----- ------ ------ -------- ---- -------- ------ ------
# 1    enet  100001     1500  -      -      -        -    -        0      0
# 101  enet  100101     610   -      -      -        -    -        0      0
# 150  enet  100150     1500  -      -      -        -    -        0      0
# 888  enet  100888     1500  -      -      -        -    -        0      0
# 1002 fddi  101002     1500  -      -      -        -    -        0      0
# 1003 trcrf 101003     4472  1005   3276   -        -    srb      0      0
# 1004 fdnet 101004     1500  -      -      -        ieee -        0      0
# 1005 trbrf 101005     4472  -      -      15       ibm  -        0      0
#
#
# VLAN AREHops STEHops Backup CRF
# ---- ------- ------- ----------
# 1003 7       7       off
#
# Remote SPAN VLANs
# ------------------------------------------------------------------------------
# 150
#
# Primary Secondary Type              Ports
# ------- --------- ----------------- ------------------------------------------
#
# vlan configuration 101
#   member evpn-instance 101 vni 10101
# vlan configuration 102
#   member evpn-instance 102 vni 10102
# vlan configuration 901
#   member vni 50901

- name: Parse the commands for provided configuration
  cisco.ios.ios_vlans:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------
#
# "parsed": [
#     {
#         "name": "default",
#         "vlan_id": 1,
#         "state": "active",
#         "shutdown": "disabled",
#         "mtu": 1500,
#     },
#     {
#         "name": "RemoteIsInMyName",
#         "vlan_id": 101,
#         "state": "active",
#         "shutdown": "enabled",
#         "mtu": 610,
#         "member": {"evi": 101, "vni": 10101},
#     },
#     {
#         "name": "VLAN0150",
#         "vlan_id": 150,
#         "state": "active",
#         "shutdown": "disabled",
#         "mtu": 1500,
#         "remote_span": True,
#     },
#     {
#         "name": "a_very_long_vlan_name_a_very_long_vlan_name",
#         "vlan_id": 888,
#         "state": "active",
#         "shutdown": "disabled",
#         "mtu": 1500,
#     },
#     {
#         "name": "fddi-default",
#         "vlan_id": 1002,
#         "state": "active",
#         "shutdown": "enabled",
#         "mtu": 1500,
#     },
#     {
#         "name": "trcrf-default",
#         "vlan_id": 1003,
#         "state": "active",
#         "shutdown": "enabled",
#         "mtu": 4472,
#     },
#     {
#         "name": "fddinet-default",
#         "vlan_id": 1004,
#         "state": "active",
#         "shutdown": "enabled",
#         "mtu": 1500,
#     },
#     {
#         "name": "trbrf-default",
#         "vlan_id": 1005,
#         "state": "active",
#         "shutdown": "enabled",
#         "mtu": 4472,
#     },
#     {"vlan_id": 102, "member": {"evi": 102, "vni": 10102}},
#     {"vlan_id": 901, "member": {"vni": 50901}},
# ]
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - vlan configuration 202
    - state active
    - remote-span
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - vlan configuration 202
    - member evpn-instance 202 vni 10202
    - vlan 200
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.vlans.vlans import (
    VlansArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.vlans.vlans import Vlans
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import get_connection


def _is_l2_device(module):
    """fails module if device is L3."""
    connection = get_connection(module)
    check_os_type = connection.get_device_info()
    if check_os_type.get("network_os_type") == "L3":
        return False
    return True


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=VlansArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "purged", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    if _is_l2_device(module) or module.params.get("state") in ["rendered", "parsed"]:
        result = Vlans(module).execute_module()
        module.exit_json(**result)
    else:
        module.fail_json("""Resource VLAN is not valid for the target device.""")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The arg spec for the iosxr_ospfv3 module
"""


class Ospfv3Args(object):  # pylint: disable=R0903
    """The arg spec for the iosxr_ospfv3 module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "running_config": {"type": "str"},
        "config": {
            "type": "dict",
            "options": {
                "processes": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "process_id": {"type": "str", "required": True},
                        "address_family_unicast": {"type": "bool"},
                        "authentication": {
                            "type": "dict",
                            "options": {
                                "disable": {"type": "bool", "default": False},
                                "ipsec": {
                                    "type": "dict",
                                    "options": {
                                        "spi": {"type": "int"},
                                        "algorithim_type": {
                                            "type": "str",
                                            "choices": ["md5", "sha1"],
                                        },
                                        "key": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                        "clear_key": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                        "password_key": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                    },
                                },
                            },
                        },
                        "auto_cost": {
                            "type": "dict",
                            "options": {
                                "reference_bandwidth": {"type": "int"},
                                "disable": {"type": "bool"},
                            },
                        },
                        "bfd": {
                            "type": "dict",
                            "options": {
                                "fast_detect": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "strict_mode": {"type": "bool"},
                                    },
                                },
                                "minimum_interval": {"type": "int"},
                                "multiplier": {"type": "int"},
                            },
                        },
                        "areas": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "area_id": {"type": "str", "required": True},
                                "authentication": {
                                    "type": "dict",
                                    "options": {
                                        "disable": {
                                            "type": "bool",
                                            "default": False,
                                        },
                                        "ipsec": {
                                            "type": "dict",
                                            "options": {
                                                "spi": {"type": "int"},
                                                "algorithim_type": {
                                                    "type": "str",
                                                    "choices": ["md5", "sha1"],
                                                },
                                                "key": {
                                                    "type": "str",
                                                    "no_log": False,
                                                },
                                                "clear_key": {
                                                    "type": "str",
                                                    "no_log": False,
                                                },
                                                "password_key": {
                                                    "type": "str",
                                                    "no_log": False,
                                                },
                                            },
                                        },
                                    },
                                },
                                "bfd": {
                                    "type": "dict",
                                    "options": {
                                        "fast_detect": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "strict_mode": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "minimum_interval": {"type": "int"},
                                        "multiplier": {"type": "int"},
                                    },
                                },
                                "cost": {"type": "int"},
                                "database_filter": {
                                    "type": "dict",
                                    "options": {
                                        "all_outgoing_lsa": {"type": "bool"},
                                    },
                                },
                                "dead_interval": {"type": "int"},
                                "default_cost": {"type": "int"},
                                "demand_circuit": {"type": "bool"},
                                "distrinbute_rib_prefix_list_name": {
                                    "type": "str",
                                },
                                "fast_reroute": {
                                    "type": "dict",
                                    "options": {
                                        "disabled": {"type": "bool"},
                                        "per_link": {
                                            "type": "dict",
                                            "options": {
                                                "information_type": {
                                                    "type": "str",
                                                    "choices": [
                                                        "exclude",
                                                        "lfa_candidate",
                                                    ],
                                                },
                                                "use_candidate_only": {
                                                    "type": "bool",
                                                },
                                                "interface": {
                                                    "type": "dict",
                                                    "options": {
                                                        "bvi": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "bundle_ether": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "pos_int": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "fast_ethernet": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "fiftygige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "fortygige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "fourhundredgige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "gigabitethernet": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "hundredgige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "mgmteth": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "multilink": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "pw_ether": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "pw_iw": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "srp": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "serial": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "tengige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "twentyfivegige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "twohundredgige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "nve": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_ip": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_ipsec": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_mte": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_mpls": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "per_prefix": {
                                            "type": "dict",
                                            "options": {
                                                "information_type": {
                                                    "type": "str",
                                                    "choices": [
                                                        "exclude",
                                                        "lfa_candidate",
                                                    ],
                                                },
                                                "use_candidate_only": {
                                                    "type": "bool",
                                                },
                                                "interface": {
                                                    "type": "dict",
                                                    "options": {
                                                        "bvi": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "bundle_ether": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "pos_int": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "fast_ethernet": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "fiftygige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "fortygige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "fourhundredgige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "gigabitethernet": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "hundredgige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "mgmteth": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "multilink": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "pw_ether": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "pw_iw": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "srp": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "serial": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "tengige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "twentyfivegige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "twohundredgige": {
                                                            "type": "list",
                                                            "elements": "str",
                                                        },
                                                        "nve": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_ip": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_ipsec": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_mte": {
                                                            "type": "list",
                                                            "elements": "int",
                                                        },
                                                        "tunnel_mpls": {
                                                            "type": "int",
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "flood_reduction": {"type": "bool"},
                                "hello_interval": {"type": "int"},
                                "instance_id": {"type": "int"},
                                "mtu_ignore": {"type": "bool"},
                                "mpls_ldp_sync": {"type": "bool"},
                                "network": {
                                    "type": "str",
                                    "choices": [
                                        "broadcast",
                                        "non-broadcast",
                                        "point-to-multipoint",
                                        "point-to-point",
                                    ],
                                },
                                "nssa": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "default_information_originate": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "metric": {"type": "int"},
                                                "metric_type": {"type": "int"},
                                            },
                                        },
                                        "no_redistribution": {"type": "bool"},
                                        "no_summary": {"type": "bool"},
                                        "translate": {
                                            "type": "dict",
                                            "options": {
                                                "type7": {
                                                    "type": "dict",
                                                    "options": {
                                                        "always": {
                                                            "type": "bool",
                                                            "required": True,
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "packet_size": {"type": "int"},
                                "passive": {"type": "bool"},
                                "prefix_suppression": {"type": "bool"},
                                "priority": {"type": "int"},
                                "ranges": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "address": {
                                            "type": "str",
                                            "required": True,
                                        },
                                        "cost": {"type": "int"},
                                        "advertise": {"type": "bool"},
                                        "not_advertise": {"type": "bool"},
                                    },
                                },
                                "retransmit_interval": {"type": "int"},
                                "stub": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "no_summary": {"type": "bool"},
                                    },
                                },
                                "transmit_delay": {"type": "int"},
                                "virtual_link": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "id": {
                                            "type": "str",
                                            "required": True,
                                        },
                                        "authentication": {
                                            "type": "dict",
                                            "options": {
                                                "disable": {
                                                    "type": "bool",
                                                    "default": False,
                                                },
                                                "ipsec": {
                                                    "type": "dict",
                                                    "options": {
                                                        "spi": {"type": "int"},
                                                        "algorithim_type": {
                                                            "type": "str",
                                                            "choices": [
                                                                "md5",
                                                                "sha1",
                                                            ],
                                                        },
                                                        "key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "clear_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "password_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "dead_interval": {"type": "int"},
                                        "hello_interval": {"type": "int"},
                                        "retransmit_interval": {"type": "int"},
                                        "transmit_delay": {"type": "int"},
                                        "encryption": {
                                            "type": "dict",
                                            "options": {
                                                "disable": {
                                                    "type": "bool",
                                                    "default": False,
                                                },
                                                "ipsec": {
                                                    "type": "dict",
                                                    "options": {
                                                        "spi": {"type": "int"},
                                                        "esp": {
                                                            "type": "dict",
                                                            "options": {
                                                                "triple_des": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "clear_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "password_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                    },
                                                                },
                                                                "aes": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "algorithim_type": {
                                                                            "type": "str",
                                                                            "choices": [
                                                                                "192",
                                                                                "256",
                                                                            ],
                                                                        },
                                                                        "key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "clear_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "password_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                    },
                                                                },
                                                                "des": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "clear_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "password_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                    },
                                                                },
                                                                "null_encryption": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "authentication": {
                                                                            "type": "dict",
                                                                            "options": {
                                                                                "algorithim_type": {
                                                                                    "type": "str",
                                                                                    "choices": [
                                                                                        "md5",
                                                                                        "sha1",
                                                                                    ],
                                                                                },
                                                                                "key": {
                                                                                    "type": "str",
                                                                                    "no_log": False,
                                                                                },
                                                                                "clear_key": {
                                                                                    "type": "str",
                                                                                    "no_log": False,
                                                                                },
                                                                                "password_key": {
                                                                                    "type": "str",
                                                                                    "no_log": False,
                                                                                },
                                                                            },
                                                                        },
                                                                    },
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "encryption": {
                                    "type": "dict",
                                    "options": {
                                        "disable": {
                                            "type": "bool",
                                            "default": False,
                                        },
                                        "ipsec": {
                                            "type": "dict",
                                            "options": {
                                                "spi": {"type": "int"},
                                                "esp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "triple_des": {
                                                            "type": "dict",
                                                            "options": {
                                                                "key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "clear_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "password_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                            },
                                                        },
                                                        "aes": {
                                                            "type": "dict",
                                                            "options": {
                                                                "algorithim_type": {
                                                                    "type": "str",
                                                                    "choices": [
                                                                        "192",
                                                                        "256",
                                                                    ],
                                                                },
                                                                "key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "clear_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "password_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                            },
                                                        },
                                                        "des": {
                                                            "type": "dict",
                                                            "options": {
                                                                "key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "clear_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "password_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                            },
                                                        },
                                                        "null_encryption": {
                                                            "type": "dict",
                                                            "options": {
                                                                "authentication": {
                                                                    "type": "dict",
                                                                    "options": {
                                                                        "algorithim_type": {
                                                                            "type": "str",
                                                                            "choices": [
                                                                                "md5",
                                                                                "sha1",
                                                                            ],
                                                                        },
                                                                        "key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "clear_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                        "password_key": {
                                                                            "type": "str",
                                                                            "no_log": False,
                                                                        },
                                                                    },
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "capability": {
                            "type": "dict",
                            "options": {
                                "type7": {
                                    "type": "dict",
                                    "options": {
                                        "prefer": {"type": "bool"},
                                        "translate": {"type": "bool"},
                                    },
                                },
                            },
                        },
                        "cost": {"type": "int"},
                        "database_filter": {
                            "type": "dict",
                            "options": {"all_outgoing_lsa": {"type": "bool"}},
                        },
                        "dead_interval": {"type": "int"},
                        "default_information_originate": {
                            "type": "dict",
                            "options": {
                                "always": {"type": "bool"},
                                "metric": {"type": "int"},
                                "metric_type": {"type": "int"},
                                "route_policy": {"type": "str"},
                                "tag": {"type": "int"},
                                "set": {"type": "bool"},
                            },
                        },
                        "default_metric": {"type": "int"},
                        "demand_circuit": {"type": "bool"},
                        "distance": {
                            "type": "dict",
                            "options": {
                                "admin_distance": {"type": "int"},
                                "ospfv3_distance": {
                                    "type": "dict",
                                    "options": {
                                        "external": {"type": "int"},
                                        "inter_area": {"type": "int"},
                                        "intra_area": {"type": "int"},
                                    },
                                },
                            },
                        },
                        "distribute_list": {
                            "type": "dict",
                            "options": {
                                "prefix_list": {
                                    "type": "list",
                                    "elements": "str",
                                    "options": {
                                        "name": {"type": "str"},
                                        "in": {"type": "bool"},
                                        "out": {"type": "bool"},
                                    },
                                },
                            },
                        },
                        "encryption": {
                            "type": "dict",
                            "options": {
                                "disable": {"type": "bool", "default": False},
                                "ipsec": {
                                    "type": "dict",
                                    "options": {
                                        "spi": {"type": "int"},
                                        "esp": {
                                            "type": "dict",
                                            "options": {
                                                "triple_des": {
                                                    "type": "dict",
                                                    "options": {
                                                        "key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "clear_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "password_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                    },
                                                },
                                                "aes": {
                                                    "type": "dict",
                                                    "options": {
                                                        "algorithim_type": {
                                                            "type": "str",
                                                            "choices": [
                                                                "192",
                                                                "256",
                                                            ],
                                                        },
                                                        "key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "clear_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "password_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                    },
                                                },
                                                "des": {
                                                    "type": "dict",
                                                    "options": {
                                                        "key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "clear_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                        "password_key": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                    },
                                                },
                                                "null_encryption": {
                                                    "type": "dict",
                                                    "options": {
                                                        "authentication": {
                                                            "type": "dict",
                                                            "options": {
                                                                "algorithim_type": {
                                                                    "type": "str",
                                                                    "choices": [
                                                                        "md5",
                                                                        "sha1",
                                                                    ],
                                                                },
                                                                "key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "clear_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                                "password_key": {
                                                                    "type": "str",
                                                                    "no_log": False,
                                                                },
                                                            },
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "fast_reroute": {
                            "type": "dict",
                            "options": {
                                "disabled": {"type": "bool"},
                                "per_link": {
                                    "type": "dict",
                                    "options": {
                                        "information_type": {
                                            "type": "str",
                                            "choices": [
                                                "exclude",
                                                "lfa_candidate",
                                            ],
                                        },
                                        "use_candidate_only": {"type": "bool"},
                                        "interface": {
                                            "type": "dict",
                                            "options": {
                                                "bvi": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "bundle_ether": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "pos_int": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "fast_ethernet": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "fiftygige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "fortygige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "fourhundredgige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "gigabitethernet": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "hundredgige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "mgmteth": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "multilink": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "pw_ether": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "pw_iw": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "srp": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "serial": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "tengige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "twentyfivegige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "twohundredgige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "nve": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_ip": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_ipsec": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_mte": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_mpls": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                                "per_prefix": {
                                    "type": "dict",
                                    "options": {
                                        "information_type": {
                                            "type": "str",
                                            "choices": [
                                                "exclude",
                                                "lfa_candidate",
                                            ],
                                        },
                                        "use_candidate_only": {"type": "bool"},
                                        "interface": {
                                            "type": "dict",
                                            "options": {
                                                "bvi": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "bundle_ether": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "post_int": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "fast_ethernet": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "fiftygige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "fortygige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "fourhundredgige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "gigabitethernet": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "hundredgige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "mgmteth": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "multilink": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "pw_ether": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "pw_iw": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "srp": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "serial": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "tengige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "twentyfivegige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "twohundredgige": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "nve": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_ip": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_ipsec": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_mte": {
                                                    "type": "list",
                                                    "elements": "int",
                                                },
                                                "tunnel_mpls": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "flood_reduction": {"type": "bool"},
                        "graceful_restart": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "helper_disable": {"type": "bool"},
                                "min_interval": {"type": "int"},
                                "max_interval": {"type": "int"},
                            },
                        },
                        "hello_interval": {"type": "int"},
                        "ignore_mospf_type6_lsa": {"type": "bool"},
                        "instance_id": {"type": "int"},
                        "log_adjacency_changes": {
                            "type": "dict",
                            "options": {
                                "set": {"type": "bool"},
                                "disable": {"type": "bool"},
                                "detail": {"type": "bool"},
                            },
                        },
                        "maximum": {
                            "type": "dict",
                            "options": {
                                "interfaces": {"type": "int"},
                                "paths": {"type": "int"},
                                "redistributed_prefixes": {"type": "int"},
                            },
                        },
                        "mpls_ldp_sync": {"type": "bool"},
                        "mtu_ignore": {"type": "bool"},
                        "network": {
                            "type": "str",
                            "choices": [
                                "broadcast",
                                "non-broadcast",
                                "point-to-multipoint",
                                "point-to-point",
                            ],
                        },
                        "nsr": {"type": "bool"},
                        "packet_size": {"type": "int"},
                        "passive": {"type": "bool"},
                        "prefix_suppression": {"type": "bool"},
                        "priority": {"type": "int"},
                        "protocol_shutdown": {"type": "bool"},
                        "redistribute": {
                            "type": "dict",
                            "options": {
                                "application": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "id": {
                                            "type": "str",
                                            "required": True,
                                        },
                                        "set": {"type": "bool"},
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "bgp": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "id": {
                                            "type": "int",
                                            "required": True,
                                        },
                                        "set": {"type": "bool"},
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "preserved_med": {"type": "str"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "connected": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "eigrp": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "id": {
                                            "type": "int",
                                            "required": True,
                                        },
                                        "set": {"type": "bool"},
                                        "match": {
                                            "type": "str",
                                            "choices": [
                                                "external",
                                                "internal",
                                            ],
                                        },
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "isis": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "id": {
                                            "type": "str",
                                            "required": True,
                                        },
                                        "set": {"type": "bool"},
                                        "level": {
                                            "type": "str",
                                            "choices": [
                                                "level-1",
                                                "level-1-2",
                                                "level-2",
                                            ],
                                        },
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "mobile": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "ospfv3": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "id": {
                                            "type": "str",
                                            "required": True,
                                        },
                                        "set": {"type": "bool"},
                                        "match": {
                                            "type": "dict",
                                            "options": {
                                                "external": {
                                                    "type": "int",
                                                    "choices": ["1", "2"],
                                                },
                                                "nssa_external": {
                                                    "type": "int",
                                                    "choices": ["1", "2"],
                                                },
                                                "internal": {"type": "bool"},
                                            },
                                        },
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "static": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                                "subscriber": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "metric": {"type": "int"},
                                        "metric_type": {"type": "int"},
                                        "route_policy": {"type": "str"},
                                        "tag": {"type": "int"},
                                    },
                                },
                            },
                        },
                        "retransmit_interval": {"type": "int"},
                        "router_id": {"type": "str"},
                        "spf_prefix_priority": {
                            "type": "dict",
                            "options": {
                                "disable": {"type": "bool"},
                                "route_policy": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "name": {"type": "str"},
                                        "value": {"type": "str"},
                                    },
                                },
                            },
                        },
                        "stub_router": {
                            "type": "dict",
                            "options": {
                                "router_lsa": {
                                    "type": "dict",
                                    "options": {
                                        "advertise_with": {
                                            "type": "str",
                                            "choices": [
                                                "max-metric",
                                                "r-bit",
                                                "v6-bit",
                                            ],
                                        },
                                        "always": {"type": "bool"},
                                        "external_lsa": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "metric": {"type": "int"},
                                            },
                                        },
                                        "include_stub": {"type": "bool"},
                                        "on_proc_migration": {"type": "int"},
                                        "on_proc_restart": {"type": "int"},
                                        "on_startup": {
                                            "type": "dict",
                                            "options": {
                                                "time": {"type": "int"},
                                                "wait_for_bgp": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "on_switchover": {"type": "int"},
                                        "summary_lsa": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "metric": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "summary_prefix": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "prefix": {"type": "str", "required": True},
                                "not_advertise": {"type": "bool"},
                                "tag": {"type": "int"},
                            },
                        },
                        "timers": {
                            "type": "dict",
                            "options": {
                                "lsa_arrival": {"type": "int"},
                                "pacing": {
                                    "type": "dict",
                                    "options": {
                                        "flood": {"type": "int"},
                                        "lsa_group": {"type": "int"},
                                        "retransmission": {"type": "int"},
                                    },
                                },
                                "throttle": {
                                    "type": "dict",
                                    "options": {
                                        "lsa": {
                                            "type": "dict",
                                            "options": {
                                                "all_lsa_initial": {
                                                    "type": "int",
                                                },
                                                "all_lsa_minimum": {
                                                    "type": "int",
                                                },
                                            },
                                        },
                                        "spf": {
                                            "type": "dict",
                                            "options": {
                                                "spf_initial": {"type": "int"},
                                                "spf_minimum": {"type": "int"},
                                            },
                                        },
                                    },
                                },
                            },
                        },
                        "trace": {
                            "type": "dict",
                            "options": {
                                "size": {"type": "str"},
                                "value": {"type": "int"},
                            },
                        },
                        "transmit_delay": {"type": "int"},
                    },
                },
            },
        },
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



# Copyright (c) 2020-2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import typing as t
from collections.abc import Callable

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.dns.plugins.module_utils._conversion.txt import (
    decode_txt_value,
    encode_txt_value,
)


def quote_txt(
    value: t.Any, always_quote: t.Any = False, character_encoding: t.Any = "decimal"
) -> str:
    if not isinstance(value, (str, bytes)):
        raise AnsibleFilterError("Input for community.dns.quote_txt must be a string")
    if not isinstance(always_quote, bool):
        raise AnsibleFilterError(
            f"always_quote must be a boolean, not {always_quote!r}"
        )
    if character_encoding not in ("decimal", "octal"):
        raise AnsibleFilterError(
            f'character_encoding must be "decimal" or "octal", not {character_encoding!r}'
        )
    value = to_text(value)
    return encode_txt_value(
        value, always_quote=always_quote, character_encoding=character_encoding
    )


def unquote_txt(value: t.Any, character_encoding: t.Any = "decimal") -> str:
    if not isinstance(value, (str, bytes)):
        raise AnsibleFilterError("Input for community.dns.unquote_txt must be a string")
    if character_encoding not in ("decimal", "octal"):
        raise AnsibleFilterError(
            f'character_encoding must be "decimal" or "octal", not {character_encoding!r}'
        )
    value = to_text(value)
    return decode_txt_value(value, character_encoding=character_encoding)


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "quote_txt": quote_txt,
            "unquote_txt": unquote_txt,
        }



#!/usr/bin/python
#
# Copyright (c) 2026 Zun Yang (@zunyangc)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_firewallpolicyrulecollectiongroup
version_added: "3.19.0"
short_description: Manage rule collection groups of an Azure Firewall Policy
description:
    - Create, update or delete a rule collection group of an Azure Firewall Policy.
    - A rule collection group hosts an ordered list of filter and/or NAT rule collections, each of which
      contains application, network or NAT rules.
options:
    resource_group:
        description:
            - Name of the resource group.
        required: true
        type: str
    firewall_policy_name:
        description:
            - The name of the parent firewall policy.
        required: true
        type: str
    name:
        description:
            - The name of the rule collection group.
        required: true
        type: str
    priority:
        description:
            - Priority of the rule collection group resource.
            - Valid values are between C(100) and C(65000).
        type: int
    rule_collections:
        description:
            - Group of firewall policy rule collections.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - Name of the rule collection.
                required: true
                type: str
            priority:
                description:
                    - Priority of the rule collection.
                    - Valid values are between C(100) and C(65000).
                type: int
            rule_collection_type:
                description:
                    - The type of the rule collection.
                required: true
                type: str
                choices:
                    - filter
                    - nat
            action:
                description:
                    - The action of the rule collection.
                    - Use C(allow) or C(deny) for filter collections.
                    - Use C(dnat) for nat collections.
                type: str
                choices:
                    - allow
                    - deny
                    - dnat
            rules:
                description:
                    - Rules included in this rule collection.
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - Name of the rule.
                        required: true
                        type: str
                    description:
                        description:
                            - Description of the rule.
                        type: str
                    rule_type:
                        description:
                            - Type of the rule.
                            - A filter collection may contain C(application) or C(network) rules.
                            - A nat collection may contain C(nat) rules.
                        required: true
                        type: str
                        choices:
                            - application
                            - network
                            - nat
                    source_addresses:
                        description:
                            - List of source IP addresses or CIDR ranges for this rule.
                        type: list
                        elements: str
                    destination_addresses:
                        description:
                            - List of destination IP addresses, CIDR ranges or service tags for this rule.
                        type: list
                        elements: str
                    source_ip_groups:
                        description:
                            - List of source IP group resource IDs for this rule.
                        type: list
                        elements: str
                    ip_protocols:
                        description:
                            - List of IP protocols for this rule.
                            - Used by C(network) and C(nat) rules only.
                        type: list
                        elements: str
                        choices:
                            - Any
                            - TCP
                            - UDP
                            - ICMP
                    destination_ports:
                        description:
                            - List of destination ports for this rule.
                            - Used by C(network) and C(nat) rules only.
                        type: list
                        elements: str
                    destination_ip_groups:
                        description:
                            - List of destination IP group resource IDs for this rule.
                            - Used by C(network) rules only.
                        type: list
                        elements: str
                    destination_fqdns:
                        description:
                            - List of destination FQDNs for this rule.
                            - Used by C(network) rules only.
                        type: list
                        elements: str
                    protocols:
                        description:
                            - List of application protocols for this rule.
                            - Used by C(application) rules only.
                        type: list
                        elements: dict
                        suboptions:
                            type:
                                description:
                                    - Protocol type.
                                type: str
                                choices:
                                    - Http
                                    - Https
                            port:
                                description:
                                    - Port number for the protocol. Valid values are between C(0) and C(64000).
                                type: int
                    target_fqdns:
                        description:
                            - List of destination FQDNs for this rule.
                            - Used by C(application) rules only.
                        type: list
                        elements: str
                    target_urls:
                        description:
                            - List of destination URLs for this rule.
                            - Used by C(application) rules only.
                        type: list
                        elements: str
                    fqdn_tags:
                        description:
                            - List of FQDN tags for this rule.
                            - Used by C(application) rules only.
                        type: list
                        elements: str
                    web_categories:
                        description:
                            - List of destination Azure web categories for this rule.
                            - Used by C(application) rules only. Requires Premium tier firewall policy.
                        type: list
                        elements: str
                    terminate_tls:
                        description:
                            - Whether to terminate TLS connections for this rule.
                            - Used by C(application) rules only. Requires Premium tier firewall policy.
                        type: bool
                    http_headers_to_insert:
                        description:
                            - List of HTTP/S headers to insert.
                            - Used by C(application) rules only.
                        type: list
                        elements: dict
                        suboptions:
                            header_name:
                                description:
                                    - The name of the header.
                                type: str
                            header_value:
                                description:
                                    - The value of the header.
                                type: str
                    translated_address:
                        description:
                            - The translated address for this NAT rule.
                            - Used by C(nat) rules only.
                        type: str
                    translated_port:
                        description:
                            - The translated port for this NAT rule.
                            - Used by C(nat) rules only.
                        type: str
                    translated_fqdn:
                        description:
                            - The translated FQDN for this NAT rule.
                            - Used by C(nat) rules only.
                        type: str
    state:
        description:
            - Assert the state of the rule collection group.
            - Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zun Yang (@zunyangc)
'''

EXAMPLES = '''
- name: Create a rule collection group with filter and NAT collections
  azure_rm_firewallpolicyrulecollectiongroup:
    resource_group: myResourceGroup
    firewall_policy_name: myFirewallPolicy
    name: myRuleCollectionGroup
    priority: 200
    rule_collections:
      - name: filterCollection
        priority: 300
        rule_collection_type: filter
        action: allow
        rules:
          - name: allowWebApp
            rule_type: application
            source_addresses:
              - 10.0.0.0/24
            protocols:
              - type: Https
                port: 443
            target_fqdns:
              - "*.microsoft.com"
          - name: allowNetwork
            rule_type: network
            ip_protocols:
              - TCP
            source_addresses:
              - 10.0.0.0/24
            destination_addresses:
              - 192.168.1.0/24
            destination_ports:
              - "443"
      - name: natCollection
        priority: 400
        rule_collection_type: nat
        action: dnat
        rules:
          - name: dnatRule
            rule_type: nat
            ip_protocols:
              - TCP
            source_addresses:
              - "*"
            destination_addresses:
              - 13.0.0.1
            destination_ports:
              - "443"
            translated_address: 10.0.0.10
            translated_port: "8443"

- name: Delete a rule collection group
  azure_rm_firewallpolicyrulecollectiongroup:
    resource_group: myResourceGroup
    firewall_policy_name: myFirewallPolicy
    name: myRuleCollectionGroup
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the rule collection group.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the rule collection group.
            returned: always
            type: str
            sample: "/subscriptions/xxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/firewallPolicies/
                     myFirewallPolicy/ruleCollectionGroups/myRuleCollectionGroup"
        name:
            description:
                - Name of the rule collection group.
            returned: always
            type: str
            sample: myRuleCollectionGroup
        priority:
            description:
                - Priority of the rule collection group.
            returned: always
            type: int
            sample: 200
        provisioning_state:
            description:
                - Provisioning state of the resource.
            returned: always
            type: str
            sample: Succeeded
        rule_collections:
            description:
                - Group of rule collections.
            returned: always
            type: list
            elements: dict
        etag:
            description:
                - Resource ETag.
            returned: always
            type: str
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: Microsoft.Network/FirewallPolicies/RuleCollectionGroups
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # Handled in azure_rm_common
    pass


# Map user-facing lowercase action names to SDK enum values.
ACTION_TO_SDK = {
    'allow': 'Allow',
    'deny': 'Deny',
    'dnat': 'DNAT',
}


http_header_spec = dict(
    header_name=dict(type='str'),
    header_value=dict(type='str'),
)

application_protocol_spec = dict(
    type=dict(type='str', choices=['Http', 'Https']),
    port=dict(type='int'),
)

rule_spec = dict(
    name=dict(type='str', required=True),
    description=dict(type='str'),
    rule_type=dict(type='str', required=True, choices=['application', 'network', 'nat']),
    source_addresses=dict(type='list', elements='str'),
    destination_addresses=dict(type='list', elements='str'),
    source_ip_groups=dict(type='list', elements='str'),
    ip_protocols=dict(type='list', elements='str', choices=['Any', 'TCP', 'UDP', 'ICMP']),
    destination_ports=dict(type='list', elements='str'),
    destination_ip_groups=dict(type='list', elements='str'),
    destination_fqdns=dict(type='list', elements='str'),
    protocols=dict(type='list', elements='dict', options=application_protocol_spec),
    target_fqdns=dict(type='list', elements='str'),
    target_urls=dict(type='list', elements='str'),
    fqdn_tags=dict(type='list', elements='str'),
    web_categories=dict(type='list', elements='str'),
    terminate_tls=dict(type='bool'),
    http_headers_to_insert=dict(type='list', elements='dict', options=http_header_spec),
    translated_address=dict(type='str'),
    translated_port=dict(type='str'),
    translated_fqdn=dict(type='str'),
)

rule_collection_spec = dict(
    name=dict(type='str', required=True),
    priority=dict(type='int'),
    rule_collection_type=dict(type='str', required=True, choices=['filter', 'nat']),
    action=dict(type='str', choices=['allow', 'deny', 'dnat']),
    rules=dict(type='list', elements='dict', options=rule_spec),
)


class AzureRMFirewallPolicyRuleCollectionGroup(AzureRMModuleBaseExt):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            firewall_policy_name=dict(type='str', required=True),
            name=dict(type='str', required=True),
            priority=dict(type='int'),
            rule_collections=dict(type='list', elements='dict', options=rule_collection_spec),
            state=dict(type='str', default='present', choices=['present', 'absent']),
        )

        self.results = dict(
            changed=False,
            state=dict(),
        )

        self.resource_group = None
        self.firewall_policy_name = None
        self.name = None
        self.priority = None
        self.rule_collections = None
        self.state = None

        super(AzureRMFirewallPolicyRuleCollectionGroup, self).__init__(self.module_arg_spec,
                                                                       supports_tags=False,
                                                                       supports_check_mode=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec.keys():
            setattr(self, key, kwargs[key])

        self._validate_input()

        changed = False
        results = dict()
        existing = self.get_rule_collection_group()

        if existing is not None:
            results = existing.as_dict()
            if self.state == 'present':
                desired = self._build_rule_collection_group()
                if self._needs_update(existing, desired):
                    changed = True
            else:
                changed = True
        else:
            if self.state == 'present':
                changed = True

        self.results['changed'] = changed
        self.results['state'] = results

        if self.check_mode:
            return self.results

        if changed:
            if self.state == 'present':
                desired = self._build_rule_collection_group()
                self.results['state'] = self.create_or_update_rule_collection_group(desired)
            else:
                self.delete_rule_collection_group()
                self.results['state'] = 'Deleted'

        return self.results

    def _validate_input(self):
        if self.rule_collections is None:
            return
        for collection in self.rule_collections:
            ctype = collection.get('rule_collection_type')
            action = collection.get('action')
            if ctype == 'filter' and action not in (None, 'allow', 'deny'):
                self.fail("rule_collection '{0}': action must be 'allow' or 'deny' for a filter collection".format(
                    collection.get('name')))
            if ctype == 'nat' and action not in (None, 'dnat'):
                self.fail("rule_collection '{0}': action must be 'dnat' for a nat collection".format(
                    collection.get('name')))
            rules = collection.get('rules') or []
            for rule in rules:
                rtype = rule.get('rule_type')
                if ctype == 'filter' and rtype == 'nat':
                    self.fail("rule '{0}': nat rules cannot belong to a filter collection".format(
                        rule.get('name')))
                if ctype == 'nat' and rtype != 'nat':
                    self.fail("rule '{0}': only nat rules can belong to a nat collection".format(
                        rule.get('name')))
            # Azure requires all rules within a single collection to be of the same type
            rule_types = {r.get('rule_type') for r in rules}
            if len(rule_types) > 1:
                self.fail(
                    "rule_collection '{0}': all rules in a single collection must be of the same type; got {1}".format(
                        collection.get('name'), sorted(rule_types)))

    def get_rule_collection_group(self):
        try:
            return self.network_client.firewall_policy_rule_collection_groups.get(
                resource_group_name=self.resource_group,
                firewall_policy_name=self.firewall_policy_name,
                rule_collection_group_name=self.name,
            )
        except ResourceNotFoundError:
            return None

    def create_or_update_rule_collection_group(self, parameters):
        try:
            response = self.network_client.firewall_policy_rule_collection_groups.begin_create_or_update(
                resource_group_name=self.resource_group,
                firewall_policy_name=self.firewall_policy_name,
                rule_collection_group_name=self.name,
                parameters=parameters,
            )
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error creating or updating Firewall Policy Rule Collection Group {0}/{1} - {2}".format(
                self.firewall_policy_name, self.name, str(exc)))
        return response.as_dict()

    def delete_rule_collection_group(self):
        try:
            response = self.network_client.firewall_policy_rule_collection_groups.begin_delete(
                resource_group_name=self.resource_group,
                firewall_policy_name=self.firewall_policy_name,
                rule_collection_group_name=self.name,
            )
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error deleting Firewall Policy Rule Collection Group {0}/{1} - {2}".format(
                self.firewall_policy_name, self.name, str(exc)))
        return response

    def _build_rule_collection_group(self):
        rule_collections = [self._build_rule_collection(c) for c in (self.rule_collections or [])]
        return self.network_models.FirewallPolicyRuleCollectionGroup(
            name=self.name,
            priority=self.priority,
            rule_collections=rule_collections or None,
        )

    def _build_rule_collection(self, collection):
        name = collection['name']
        priority = collection.get('priority')
        rules = [self._build_rule(r) for r in (collection.get('rules') or [])]
        ctype = collection['rule_collection_type']
        action_value = ACTION_TO_SDK.get(collection.get('action')) if collection.get('action') else None

        if ctype == 'filter':
            action = self.network_models.FirewallPolicyFilterRuleCollectionAction(type=action_value) \
                if action_value else None
            return self.network_models.FirewallPolicyFilterRuleCollection(
                name=name,
                priority=priority,
                action=action,
                rules=rules or None,
            )
        action = self.network_models.FirewallPolicyNatRuleCollectionAction(type=action_value) \
            if action_value else None
        return self.network_models.FirewallPolicyNatRuleCollection(
            name=name,
            priority=priority,
            action=action,
            rules=rules or None,
        )

    def _build_rule(self, rule):
        rtype = rule['rule_type']
        common = dict(name=rule['name'], description=rule.get('description'))

        if rtype == 'application':
            protocols = None
            if rule.get('protocols'):
                protocols = [
                    self.network_models.FirewallPolicyRuleApplicationProtocol(
                        protocol_type=p.get('type'),
                        port=p.get('port'),
                    ) for p in rule['protocols']
                ]
            headers = None
            if rule.get('http_headers_to_insert'):
                headers = [
                    self.network_models.FirewallPolicyHttpHeaderToInsert(
                        header_name=h.get('header_name'),
                        header_value=h.get('header_value'),
                    ) for h in rule['http_headers_to_insert']
                ]
            return self.network_models.ApplicationRule(
                source_addresses=rule.get('source_addresses'),
                destination_addresses=rule.get('destination_addresses'),
                protocols=protocols,
                target_fqdns=rule.get('target_fqdns'),
                target_urls=rule.get('target_urls'),
                fqdn_tags=rule.get('fqdn_tags'),
                source_ip_groups=rule.get('source_ip_groups'),
                terminate_tls=rule.get('terminate_tls'),
                web_categories=rule.get('web_categories'),
                http_headers_to_insert=headers,
                **common
            )
        if rtype == 'network':
            return self.network_models.NetworkRule(
                ip_protocols=rule.get('ip_protocols'),
                source_addresses=rule.get('source_addresses'),
                destination_addresses=rule.get('destination_addresses'),
                destination_ports=rule.get('destination_ports'),
                source_ip_groups=rule.get('source_ip_groups'),
                destination_ip_groups=rule.get('destination_ip_groups'),
                destination_fqdns=rule.get('destination_fqdns'),
                **common
            )
        return self.network_models.NatRule(
            ip_protocols=rule.get('ip_protocols'),
            source_addresses=rule.get('source_addresses'),
            destination_addresses=rule.get('destination_addresses'),
            destination_ports=rule.get('destination_ports'),
            translated_address=rule.get('translated_address'),
            translated_port=rule.get('translated_port'),
            source_ip_groups=rule.get('source_ip_groups'),
            translated_fqdn=rule.get('translated_fqdn'),
            **common
        )

    def _needs_update(self, existing, desired):
        '''
        Compare the server-returned model against the user-built desired model.
        '''
        existing_dict = existing.as_dict()
        for key in ('id', 'etag', 'type', 'provisioning_state', 'size'):
            existing_dict.pop(key, None)
        desired_dict = desired.as_dict()
        return not self.default_compare({}, desired_dict, existing_dict, '', dict(compare=[]))


def main():
    AzureRMFirewallPolicyRuleCollectionGroup()


if __name__ == '__main__':
    main()



# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Felix Fontein <felix@fontein.de>

# /// script
# dependencies = ["nox>=2025.02.09", "antsibull-nox"]
# ///

from __future__ import annotations

import os
import sys
import urllib.request
from pathlib import Path

import nox

try:
    import antsibull_nox
    import antsibull_nox.sessions
except ImportError:
    print("You need to install antsibull-nox in the same Python environment as nox.")
    sys.exit(1)


IN_CI = os.environ.get("CI") == "true"


antsibull_nox.load_antsibull_nox_toml()


@nox.session(name="update-docs-fragments", default=True)
def update_docs_fragments(session: nox.Session) -> None:
    """
    Update/check auto-generated parts of docs fragments.
    """
    session.install("ansible-core")
    prepare = antsibull_nox.sessions.prepare_collections(
        session, install_in_site_packages=True
    )
    if not prepare:
        return
    data = ["python", "tests/update-docs-fragments.py"]
    if IN_CI:
        data.append("--lint")
    session.run(*data)


@nox.session(name="update-psl", default=False, python=False)
def update_psl(session: nox.Session) -> None:
    # Sometimes the version on publicsuffix.org differs depending on from where you request it over many hours,
    # so for now let's directly fetch it from GitHub.
    # url = 'https://publicsuffix.org/list/public_suffix_list.dat'
    url = "https://raw.githubusercontent.com/publicsuffix/list/main/public_suffix_list.dat"
    filename = "plugins/public_suffix_list.dat"

    # Download file
    urllib.request.urlretrieve(url, filename)

    output = session.run("git", "status", "--porcelain=v1", filename, silent=True)
    if output is None:
        # The run was skipped
        return
    if output == "":
        print("PSL is up-to-date!")
        return

    if IN_CI:
        session.error("PSL is not up-to-date! Run 'nox -e update-psl'!")
        return

    fragment = Path("changelogs", "fragments", "update-psl.yml")
    if not fragment.exists():
        fragment.write_text(r"""bugfixes:
  - "Update Public Suffix List."
""")

    session.run("git", "status", filename, fragment)


# Allow to run the noxfile with `python noxfile.py`, `pipx run noxfile.py`, or similar.
# Requires nox >= 2025.02.09
if __name__ == "__main__":
    nox.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ramses Smeyers <rsmeyers@cisco.com>
# Copyright: (c) 2023, Shreyas Srish <ssrish@cisco.com>
# Copyright: (c) 2024, Akini Ross <akinross@cisco.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
    name: aci_listify
    short_description: Flattens the nested dictionaries representing the ACI model data.
    description:
      - This filter flattens and transforms the input data into a list.
      - See the Examples section below.
    options:
      data:
        description: This option represents the ACI model data which is a list of dictionaries or a dictionary with any level of nesting data.
        type: raw
        required: True
      keys:
        description: Comma separated keys of type string denoting the ACI objects.
        required: True
"""

EXAMPLES = r"""
- name: Set vars
  ansible.builtin.set_fact:
    data:
      tenant:
        - name: ansible_test
          description: Created using listify
          app:
            - name: app_test
              epg:
                - name: web
                  bd: web_bd
                - name: app
                  bd: app_bd
          bd:
            - name: bd_test
              subnet:
                - name: 10.10.10.1
                  mask: 24
                  scope:
                    - public
                    - shared
              vrf: vrf_test
            - name: bd_test2
              subnet:
                - name: 20.20.20.1
                  mask: 24
                  scope: public
              vrf: vrf_test
          vrf:
            - name: vrf_test
          policies:
            protocol:
              bfd:
                - name: BFD-ON
                  description: Enable BFD
                  admin_state: enabled
                  detection_multiplier: 3
                  min_tx_interval: 50
                  min_rx_interval: 50
                  echo_rx_interval: 50
                  echo_admin_state: enabled
                  sub_interface_optimization_state: enabled
              ospf:
                interface:
                  - name: OSPF-P2P-IntPol
                    network_type: p2p
                    priority: 1
                  - name: OSPF-Broadcast-IntPol
                    network_type: bcast
                    priority: 1

- name: Create tenants
  cisco.aci.aci_tenant:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: '{{ item.tenant_name }}'
    description: '{{ item.tenant_description }}'
  with_items: '{{ data|cisco.aci.aci_listify("tenant") }}'

- name: Create VRFs
  cisco.aci.aci_vrf:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: '{{ item.tenant_name }}'
    vrf_name: '{{ item.tenant_vrf_name }}'
  with_items: '{{ data|cisco.aci.aci_listify("tenant","vrf") }}'

- name: Create BDs
  cisco.aci.aci_bd:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: '{{ item.tenant_name }}'
    vrf: '{{ item.tenant_bd_vrf }}'
    bd: '{{ item.tenant_bd_name }}'
    enable_routing: 'yes'
  with_items: '{{ data|cisco.aci.aci_listify("tenant","bd") }}'

- name: Create BD subnets
  cisco.aci.aci_bd_subnet:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: '{{ item.tenant_name }}'
    bd: '{{ item.tenant_bd_name }}'
    gateway: '{{ item.tenant_bd_subnet_name }}'
    mask: '{{ item.tenant_bd_subnet_mask }}'
    scope: '{{ item.tenant_bd_subnet_scope }}'
  with_items: '{{ data|cisco.aci.aci_listify("tenant","bd","subnet") }}'

- name: Create APs
  cisco.aci.aci_ap:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: '{{ item.tenant_name }}'
    app_profile: '{{ item.tenant_app_name }}'
  with_items: '{{ data|cisco.aci.aci_listify("tenant","app") }}'

- name: Create EPGs
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: '{{ item.tenant_name }}'
    app_profile: '{{ item.tenant_app_name }}'
    epg: '{{ item.tenant_app_epg_name }}'
    bd: '{{ item.tenant_app_epg_bd }}'
  with_items: '{{ data|cisco.aci.aci_listify("tenant","app","epg") }}'
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


# This function takes a dictionary and a series of keys,
# and returns a list of dictionaries using recursive helper function 'listify_worker'
def listify(d, *keys):
    return list(listify_worker(d, keys, 0, {}, ""))


# This function walks through a dictionary 'd', depth-first,
# using the keys provided, and generates a new dictionary for each key:value pair encountered
def listify_worker(d, keys, depth, cache, prefix):
    # The prefix in the code is used to store the path of keys traversed in the nested dictionary,
    # which helps to generate unique keys for each value when flattening the dictionary.
    prefix += keys[depth] + "_"

    if keys[depth] in d:
        for item in d[keys[depth]]:
            cache_work = cache.copy()
            if isinstance(item, dict):
                for k, v in item.items():
                    if isinstance(v, list) and all(isinstance(x, (str, int, float, bool, bytes)) for x in v) or not isinstance(v, (dict, list)):
                        # The cache in this code is a temporary storage that holds key-value pairs as the function navigates through the nested dictionary.
                        # It helps to generate the final output by remembering the traversed path in each recursive call.
                        cache_key = prefix + k
                        cache_value = v
                        cache_work[cache_key] = cache_value
                # If we're at the deepest level of keys
                if len(keys) - 1 == depth:
                    yield cache_work
                else:
                    for k, v in item.items():
                        if k == keys[depth + 1] and isinstance(v, (dict, list)):
                            yield from listify_worker({k: v}, keys, depth + 1, cache_work, prefix)
            # Conditional to support nested dictionaries which are detected by item is string
            elif isinstance(item, str) and isinstance(d[keys[depth]], dict):
                yield from listify_worker({item: d[keys[depth]][item]}, keys, depth + 1, cache_work, prefix)


class FilterModule(object):
    """Ansible core jinja2 filters"""

    def filters(self):
        return {
            "aci_listify": listify,
        }



#
# -*- coding: utf-8 -*-
# Copyright 2019 Cisco and/or its affiliates.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The nxos telemetry utility library
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy


def get_module_params_subsection(module_params, tms_config, resource_key=None):
    """
    Helper method to get a specific module_params subsection
    """
    mp = {}
    if tms_config == "TMS_GLOBAL":
        relevant_keys = [
            "certificate",
            "compression",
            "source_interface",
            "vrf",
        ]
        for key in relevant_keys:
            mp[key] = module_params[key]

    if tms_config == "TMS_DESTGROUP":
        mp["destination_groups"] = []
        for destgrp in module_params["destination_groups"]:
            if destgrp["id"] == resource_key:
                mp["destination_groups"].append(destgrp)

    if tms_config == "TMS_SENSORGROUP":
        mp["sensor_groups"] = []
        for sensor in module_params["sensor_groups"]:
            if sensor["id"] == resource_key:
                mp["sensor_groups"].append(sensor)

    if tms_config == "TMS_SUBSCRIPTION":
        mp["subscriptions"] = []
        for sensor in module_params["subscriptions"]:
            if sensor["id"] == resource_key:
                mp["subscriptions"].append(sensor)

    return mp


def valiate_input(playvals, type, module):
    """
    Helper method to validate playbook values for destination groups
    """
    if type == "destination_groups":
        if not playvals.get("id"):
            msg = "Invalid playbook value: {0}.".format(playvals)
            msg += " Parameter <id> under <destination_groups> is required"
            module.fail_json(msg=msg)
        if playvals.get("destination") and not isinstance(
            playvals["destination"],
            dict,
        ):
            msg = "Invalid playbook value: {0}.".format(playvals)
            msg += " Parameter <destination> under <destination_groups> must be a dict"
            module.fail_json(msg=msg)
        if not playvals.get("destination") and len(playvals) > 1:
            msg = "Invalid playbook value: {0}.".format(playvals)
            msg += " Playbook entry contains unrecongnized parameters."
            msg += (
                " Make sure <destination> keys under <destination_groups> are specified as follows:"
            )
            msg += " destination: {ip: <ip>, port: <port>, protocol: <prot>, encoding: <enc>}}"
            module.fail_json(msg=msg)

    if type == "sensor_groups":
        if not playvals.get("id"):
            msg = "Invalid playbook value: {0}.".format(playvals)
            msg += " Parameter <id> under <sensor_groups> is required"
            module.fail_json(msg=msg)
        if playvals.get("path") and "name" not in playvals["path"].keys():
            msg = "Invalid playbook value: {0}.".format(playvals)
            msg += " Parameter <path> under <sensor_groups> requires <name> key"
            module.fail_json(msg=msg)


def get_instance_data(key, cr_key, cr, existing_key):
    """
    Helper method to get instance data used to populate list structure in config
    fact dictionary
    """
    data = {}
    if existing_key is None:
        instance = None
    else:
        instance = cr._ref[cr_key]["existing"][existing_key]

    patterns = {
        "destination_groups": r"destination-group (\S+)",
        "sensor_groups": r"sensor-group (\S+)",
        "subscriptions": r"subscription (\S+)",
    }
    if key in patterns.keys():
        m = re.search(patterns[key], cr._ref["_resource_key"])
        instance_key = m.group(1)
        data = {"id": instance_key, cr_key: instance}

    # Remove None values
    data = dict((k, v) for k, v in data.items() if v is not None)
    return data


def cr_key_lookup(key, mo):
    """
    Helper method to get instance key value for Managed Object (mo)
    """
    cr_keys = [key]
    if key == "destination_groups" and mo == "TMS_DESTGROUP":
        cr_keys = ["destination"]
    elif key == "sensor_groups" and mo == "TMS_SENSORGROUP":
        cr_keys = ["data_source", "path"]
    elif key == "subscriptions" and mo == "TMS_SUBSCRIPTION":
        cr_keys = ["destination_group", "sensor_group"]

    return cr_keys


def normalize_data(cmd_ref):
    """Normalize playbook values and get_existing data"""

    playval = cmd_ref._ref.get("destination").get("playval")
    existing = cmd_ref._ref.get("destination").get("existing")

    dest_props = ["protocol", "encoding"]
    if playval:
        for prop in dest_props:
            for key in playval.keys():
                playval[key][prop] = playval[key][prop].lower()
    if existing:
        for key in existing.keys():
            for prop in dest_props:
                existing[key][prop] = existing[key][prop].lower()


def remove_duplicate_context(cmds):
    """Helper method to remove duplicate telemetry context commands"""
    if not cmds:
        return cmds
    feature_indices = [i for i, x in enumerate(cmds) if x == "feature telemetry"]
    telemetry_indices = [i for i, x in enumerate(cmds) if x == "telemetry"]
    if len(feature_indices) == 1 and len(telemetry_indices) == 1:
        return cmds
    if len(feature_indices) == 1 and not telemetry_indices:
        return cmds
    if len(telemetry_indices) == 1 and not feature_indices:
        return cmds
    if feature_indices and feature_indices[-1] > 1:
        cmds.pop(feature_indices[-1])
        return remove_duplicate_context(cmds)
    if telemetry_indices and telemetry_indices[-1] > 1:
        cmds.pop(telemetry_indices[-1])
        return remove_duplicate_context(cmds)


def get_setval_path(module_or_path_data):
    """Build setval for path parameter based on playbook inputs
    Full Command:
      - path {name} depth {depth} query-condition {query_condition} filter-condition {filter_condition}
    Required:
      - path {name}
    Optional:
      - depth {depth}
      - query-condition {query_condition},
      - filter-condition {filter_condition}
    """
    if isinstance(module_or_path_data, dict):
        path = module_or_path_data
    else:
        path = module_or_path_data.params["config"]["sensor_groups"][0].get("path")
    if path is None:
        return path

    setval = "path {name}"
    if "depth" in path.keys():
        if path.get("depth") != "None":
            setval = setval + " depth {depth}"
    if "query_condition" in path.keys():
        if path.get("query_condition") != "None":
            setval = setval + " query-condition {query_condition}"
    if "filter_condition" in path.keys():
        if path.get("filter_condition") != "None":
            setval = setval + " filter-condition {filter_condition}"

    return setval


def remove_duplicate_commands(commands_list):
    # Remove any duplicate commands.
    # pylint: disable=unnecessary-lambda
    return sorted(set(commands_list), key=lambda x: commands_list.index(x))


def massage_data(have_or_want):
    # Massage non global into a data structure that is indexed by id and
    # normalized for destination_groups, sensor_groups and subscriptions.
    data = deepcopy(have_or_want)
    massaged = {}
    massaged["destination_groups"] = {}
    massaged["sensor_groups"] = {}
    massaged["subscriptions"] = {}

    for subgroup in ["destination_groups", "sensor_groups", "subscriptions"]:
        for item in data.get(subgroup, []):
            id = str(item.get("id"))
            if id not in massaged[subgroup].keys():
                massaged[subgroup][id] = []
            item.pop("id")
            if not item:
                item = None
            else:
                if item.get("destination"):
                    if item.get("destination").get("port"):
                        item["destination"]["port"] = str(item["destination"]["port"])
                    if item.get("destination").get("protocol"):
                        item["destination"]["protocol"] = item["destination"]["protocol"].lower()
                    if item.get("destination").get("encoding"):
                        item["destination"]["encoding"] = item["destination"]["encoding"].lower()
                if item.get("path"):
                    for key in [
                        "filter_condition",
                        "query_condition",
                        "depth",
                    ]:
                        if item.get("path").get(key) == "None":
                            del item["path"][key]
                    if item.get("path").get("depth") is not None:
                        item["path"]["depth"] = str(item["path"]["depth"])
                if item.get("destination_group"):
                    item["destination_group"] = str(item["destination_group"])
                if item.get("sensor_group"):
                    if item.get("sensor_group").get("id"):
                        item["sensor_group"]["id"] = str(item["sensor_group"]["id"])
                    if item.get("sensor_group").get("sample_interval"):
                        item["sensor_group"]["sample_interval"] = str(
                            item["sensor_group"]["sample_interval"],
                        )
                if item.get("destination_group") and item.get("sensor_group"):
                    item_copy = deepcopy(item)
                    del item_copy["sensor_group"]
                    del item["destination_group"]
                    massaged[subgroup][id].append(item_copy)
                    massaged[subgroup][id].append(item)
                    continue
                if item.get("path") and item.get("data_source"):
                    item_copy = deepcopy(item)
                    del item_copy["data_source"]
                    del item["path"]
                    massaged[subgroup][id].append(item_copy)
                    massaged[subgroup][id].append(item)
                    continue
            massaged[subgroup][id].append(item)
    return massaged
